package hr.fer.tel.ruazosa.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.net.URI;
import java.util.List;

import hr.fer.tel.ruazosa.entities.Course;
import hr.fer.tel.ruazosa.entities.Person;
import hr.fer.tel.ruazosa.representations.CourseAssembler;
import hr.fer.tel.ruazosa.representations.CourseResource;
import hr.fer.tel.ruazosa.representations.CourseResourceRequest;
import hr.fer.tel.ruazosa.representations.PersonAssembler;
import hr.fer.tel.ruazosa.representations.ShortCourseResource;
import hr.fer.tel.ruazosa.representations.ShortPersonResource;
import hr.fer.tel.ruazosa.services.CourseService;
import hr.fer.tel.ruazosa.services.PersonService;

import static hr.fer.tel.ruazosa.representations.RepresentationUtility.badRequestIfNull;
import static hr.fer.tel.ruazosa.representations.RepresentationUtility.resourceNotFoundIfNull;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

@RestController
@RequestMapping(value = "/api/courses")
public class CourseController {
    private CourseService courseService;
    private CourseAssembler courseAssembler;
    private PersonAssembler personAssembler;

    @Autowired
    public CourseController(CourseService courseService, CourseAssembler courseAssembler, PersonAssembler personAssembler) {
        this.courseService = courseService;
        this.courseAssembler = courseAssembler;
        this.personAssembler = personAssembler;
    }

    @RequestMapping(method = RequestMethod.GET)
    public List<ShortCourseResource> getPersons() {
        return courseAssembler.toListResource(courseService.findAll());
    }

    @RequestMapping(value = "/{courseId}", method = RequestMethod.GET)
    public CourseResource getCourse(@PathVariable("courseId") Long courseId) {
        Course course = resourceNotFoundIfNull(courseService.getWithId(courseId));
        return courseAssembler.toResource(course);
    }

    @RequestMapping(method = RequestMethod.POST)
    public ResponseEntity newCourse(@RequestBody CourseResourceRequest courseResource) {
        Course course = courseService.save(courseAssembler.toEntity(courseResource));

        return ResponseEntity.created(
                linkTo(methodOn(CourseController.class).getCourse(course.getId())).toUri())
                .build();
    }

    @RequestMapping(value = "/{courseId}", method = RequestMethod.PUT)
    public ResponseEntity updateCourse(@PathVariable("courseId") Long id, @RequestBody CourseResourceRequest courseResourceRequest) {
        try {
            Course course = courseAssembler.toEntity(courseResourceRequest);
            course.setId(id);
            courseService.update(course);
            return ResponseEntity.noContent().build();
        } catch (IllegalArgumentException e) {
            return ResponseEntity.status(HttpStatus.NOT_MODIFIED)
                    .eTag(e.getMessage())
                    .build();
        }
    }

    @RequestMapping(value = "/{courseId}", method = RequestMethod.DELETE)
    public ResponseEntity deleteCourse(@PathVariable("courseId") Long id) {
        resourceNotFoundIfNull(courseService.getWithId(id));
        courseService.delete(id);
        return ResponseEntity.noContent().build();
    }

    @RequestMapping(value = "/{cid}/enrollPerson/{sid}", method = RequestMethod.POST)
    public ResponseEntity enrollPersonInCourse(@PathVariable("cid") Long courseId, @PathVariable("sid") Long studentId) {
        resourceNotFoundIfNull(courseId, "Course ID can not be null.");
        resourceNotFoundIfNull(studentId, "Student ID can not be null.");
        try {
            courseService.enrollPersonToCourse(courseId, studentId);
        } catch (IllegalArgumentException e) {
            badRequestIfNull(null, e.getMessage());
        }

        return ResponseEntity.noContent().build();
    }

    @RequestMapping(value = "/{cid}/unenrollPerson/{sid}", method = RequestMethod.POST)
    public ResponseEntity unenrollPersonInCourse(@PathVariable("cid") Long courseId, @PathVariable("sid") Long studentId) {
        resourceNotFoundIfNull(courseId, "Course ID can not be null.");
        resourceNotFoundIfNull(studentId, "Student ID can not be null.");
        try {
            courseService.unenrollPersonToCourse(courseId, studentId);
        } catch (IllegalArgumentException e) {
            badRequestIfNull(null, e.getMessage());
        }

        return ResponseEntity.noContent().build();
    }

    @RequestMapping(value = "/{cid}/students", method = RequestMethod.GET)
    public List<ShortPersonResource> getStudents(@PathVariable("cid") Long courseId) {
        Course course = resourceNotFoundIfNull(courseService.getWithId(courseId));
        return personAssembler.toListResource(course.getStudents());
    }
}
