package hr.fer.tel.ruazosa.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import hr.fer.tel.ruazosa.entities.Person;
import hr.fer.tel.ruazosa.representations.CourseAssembler;
import hr.fer.tel.ruazosa.representations.PersonAssembler;
import hr.fer.tel.ruazosa.representations.PersonResource;
import hr.fer.tel.ruazosa.representations.ShortCourseResource;
import hr.fer.tel.ruazosa.representations.ShortPersonResource;
import hr.fer.tel.ruazosa.services.PersonService;

import static hr.fer.tel.ruazosa.representations.RepresentationUtility.resourceNotFoundIfNull;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

@RestController
@RequestMapping(value = "/api/persons")
public class PresonController {
    private PersonService service;
    private PersonAssembler assembler;
    private CourseAssembler courseAssembler;

    @Autowired
    public PresonController(PersonService service, PersonAssembler assembler, CourseAssembler courseAssembler) {
        this.service = service;
        this.assembler = assembler;
        this.courseAssembler = courseAssembler;
    }

    @RequestMapping(method = RequestMethod.GET)
    public List<ShortPersonResource> getPersons() {
        return assembler.toListResource(service.findAll());
    }

    @RequestMapping(value = "/{personId}", method = RequestMethod.GET)
    public PersonResource getPerson(@PathVariable("personId") Long personId) {
        Person person = resourceNotFoundIfNull(service.getWithId(personId));
        return assembler.toResource(person);
    }

    @RequestMapping(method = RequestMethod.POST)
    public ResponseEntity newPerson(@RequestBody PersonResource personResource) {
        Person person = service.save(assembler.toEntity(personResource));

        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.LOCATION, linkTo(methodOn(PresonController.class).getPerson(person.getId())).toString());
        ResponseEntity response = new ResponseEntity(headers, HttpStatus.CREATED);
        return response;
    }

    @RequestMapping(value = "/{personId}", method = RequestMethod.PUT)
    public ResponseEntity updatePerson(@PathVariable("personId") Long id, @RequestBody PersonResource personResource) {
        try {
            Person person = assembler.toEntity(personResource);
            person.setId(id);
            service.update(person);
            return ResponseEntity.noContent().build();
        } catch (IllegalArgumentException e) {
            return ResponseEntity.status(HttpStatus.NOT_MODIFIED)
                    .eTag(e.getMessage())
                    .build();
        }
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public ResponseEntity deletePerson(@PathVariable("id") String id) {
        service.delete(id);
        return ResponseEntity.noContent().build();
    }

    @RequestMapping(value = "/{personId}/teachingCourses", method = RequestMethod.GET)
    public List<ShortCourseResource> getTeachingCourses(@PathVariable("personId") Long personId) {
        Person person = resourceNotFoundIfNull(service.getWithId(personId));
        return courseAssembler.toListResource(person.getTeachingCourses());
    }

    @RequestMapping(value = "/{personId}/enrolledCourses", method = RequestMethod.GET)
    public List<ShortCourseResource> getEnrolledCourses(@PathVariable("personId") Long personId) {
        Person person = resourceNotFoundIfNull(service.getWithId(personId));
        return courseAssembler.toListResource(person.getEnrolledCourses());
    }

}